/*  $Id: _stdinc.h,v 1.2 1999/07/16 20:18:50 shaggy Exp $
 *	Standard definitions included by all Posix header files
 *      relative path: ./lib/posix
 */

#ifndef __stdinc_h__
#define __stdinc_h__

/* All definitions in this file have a preceding underscore to keep
 * them in an implementation-defined namespace.
 * To bring these definitions into the regular (non-underscore) namespace,
 * the appropriate headers should be included.  Application programs 
 * should never refer to the names in this file directly.
 */

#include <btypes.h>
#include "_envtypes.h"

/* Note: all of the following string-related type definitions still
 * require the use of a '*' in the code that uses them, to make it
 * obvious that they are pointers.
 * They are provided as a way to help make it obvious how a piece of code
 * is using an array of characters (as a fixed-length array, without
 * a termination character, or as a zero-terminated string)
 * by showing the intention, rather than just the implementation.
 * This helps to reduce the chances of errors now, and will
 * help simplify the future conversion of old ASCII-based code
 * to UTF8, Unicode, etc, by providing a pattern that is easy to search for.
 * This could also aid in the future conversion to C++ string classes.
 * (note that there is also a "sometimes zero-terminated" string-- used
 * in fixed length buffers, where the string is allowed to use the full
 * size of the buffer and not have a terminating null in that case.
 * Unless a separate type is created, code use the '*sz' types for such
 * strings, to show that null is used as a termination character.)
 */

typedef char _sz;		/* string, zero-terminated */
typedef const char _csz;	/* constant string, zero-terminated */
typedef char _msz;		/* multibyte string, zero-terminated */
typedef const char _mcsz;	/* multibyte constant string, zero-terminated */
typedef _wchar_t _wsz;		/* wide string, zero-terminated */
typedef const _wchar_t _wcsz;	/* wide constant string, zero-terminated */

#ifdef __cplusplus
#	define _extern_C	extern "C"
#	define _extern_C_begin	extern "C" {
#	define _extern_C_end	}
#else
#	define _extern_C	extern
#	define _extern_C_begin
#	define _extern_C_end
#endif

#endif
