/*  $Id: chunk.h,v 1.9 1999/12/18 21:26:52 shaggy Exp $
 *      definition of chunks in the mapfile
 *      relative path: ./sys/boot/m-pc
 */

#ifdef ASM

#define Chunk_sector		0	// except two high bits
#define Chunk_cyl		1	// low eight bits 
#define Chunk_head		2	// head
#define Chunk_count		3	// 7:partial, 6:small, 6-0: count
#define Chunk_sizeof		4	// size of 'Chunk'

#else

#include <stdx1.h>

typedef struct Chunk Chunk;
struct Chunk {
	uint	sector: 6;	// CL:0-5  sector number
	uint	cyl2: 2;	// CL:6-7  cylinder (bits 8-9)
	uint	cyl1: 8;	// CH:0-7  cylinder (bits 0-7)
	uint	head: 8;	// DH:0-7  head (bits 0-7)
	uint	count: 7;	// AL:0-6  sector count
	uint	partial: 1;	// AL:7  for partial sectors
};

typedef struct ChunkLinear ChunkLinear;
struct ChunkLinear {
	uint	sector: 24;	// linear sector number 
	uint	count: 7;	// sector count
	uint	partial: 1;	// for partial sectors
};

// if chunk.partial:
//	if chunk.small (bit 6):
//		bits 0-4: start, in multiple of 16
//		bit 5: unused
//	else
//		bits 0-3: start, in multiple of 32
//		bits 4-5: select size: 32,64,128,256
// else
//	bits 0-6: number of consecutive sectors

// note:
//	'partial' chunks are for filesystems that can store multiple 
//	logical blocks in a sector (to save space for many small files)
//	only block sizes of 16(small),32,64,128 or 256 are supported
//	for partial sectors
//	They are currently not supported by the boot code, as
//	they are not yet needed, but such a mode may
//	be available in the shagfs file system, to save major
//	amounts of space in filesystems that consist of many small objects.

// large sector count extension (not fully supported yet):
//
// if linearsector >= 0x00ffc000
//		then linearsector = ((linearsector&0x3fff)<<32)|next32bits;
//		(total 2^46 sectors, or 32PB)
// if cylinder == 1023
//		then cylinder = next32bits
//		(potential total of 2^46 sectors, or 32PB)
#endif

