/*  $Id: shuffle.cc,v 1.11 1999/12/18 21:28:08 shaggy Exp $
 *      An example standalone program that shuffles the video display
 */

#define shuffle_VER_MAJOR	0
#define shuffle_VER_MINOR	1
#define shuffle_VER_PATCH	0
#define shuffle_VER_STR	"1.0"
#define shuffle_VER_DATE	"October 05, 1996"
#define shuffle_VER_NDATE	"10/05/96"

#include <libc.h>
#include <stdx1.h>
#include <defines>

static csz rcsid[] __attribute__((unused))
	= "$Id: shuffle.cc,v 1.11 1999/12/18 21:28:08 shaggy Exp $"
	  "$Release: " shuffle_VER_STR " $";


int
rnd(int max)
{
	static ulong seed = 0;
	seed = seed*1103515245+12345;
	return (int)((seed/65536)%max);
}

void
shuffle(uint16 *vbase, int wid, int hgt, int delay, int loops)
{
// Note: Due to the nature of this code (writing directly to video
// memory) it cannot be run using SOLO emulation, unless that emulation
// actually maps in the low 1MB of memory, as SOLO does.

	for(int i=0; i<wid*8; i++)
		vbase[i] |= i<<8;
	while(loops-- > 0) {
		int x1 = rnd(wid);
		int y1 = rnd(hgt);
		int x2 = (x1+rnd(3)-1+wid)%wid;
		int y2 = (y1+rnd(3)-1+hgt)%hgt;
		uint16 *p1 = vbase+(y1*wid+x1);
		uint16 *p2 = vbase+(y2*wid+x2);
		// if there is an empty space, move into it
		if(!*p2 || *p2 == 0x0720) {
			uint16 t = *p2;
			*p2 = *p1;
			*p1 = t;
		} else if(!*p1 || *p1 == 0x0720) {
			uint16 t = *p1;
			*p1 = *p2;
			*p2 = t;
		}
		for(int i=0; i<delay; i++)
			continue;
	}	
}


int
getenv_int(csz *name)
{
	csz *s = getenv(name);
	if(!s) {
		printf("error: missing '%s' in environment\n", name);
		exit(1);
	}
	return strtol(s,0,0);
}


int
main(int argc, csz **argv)
{
	if(argc != 3) {
		printf("usage: %s <delay factor> <# of loops>\n",
			argv[0]);
		exit(1);
	}
	shuffle((ushort*)getenv_int("vidbase"),
		getenv_int("vidwid"), getenv_int("vidhgt"),
		atoi(argv[1]), atoi(argv[2]));
}

