/*  $Id: unzip.cc,v 1.9 2000/01/12 04:15:58 shaggy Exp $
 */

#define unzip_VER_MAJOR	0
#define unzip_VER_MINOR	1
#define unzip_VER_PATCH	0
#define unzip_VER_STR	"0.1.0"
#define unzip_VER_DATE	"May 29, 1999"
#define unzip_VER_NDATE	"05/29/99"


#include <libc_posix.h>
#include <stdx1.h>
#include <defines>
#include <boot/libsolo.h>
#include <boot/unistd.h>
#include <boot/bootparam.h>
#include <hw/Port.h>

typedef solo_fd_t fd_t;

extern "C" {
#include "inflate.h"
#include "crc.h"

long initial_stack_size = 128*_K_;
}


static csz rcsid[] __attribute__((unused))
	= "$Id: unzip.cc,v 1.9 2000/01/12 04:15:58 shaggy Exp $"
	  "$Release: " unzip_VER_STR " $";


extern BootParam *bootparam;

typedef struct Ibuf Ibuf;
struct Ibuf {
	fd_t fd;
	int bp;
	int blen;
	byte buf[65536];
};

struct inflate *gp;

static int
lgetc(Ibuf *i)
{
	if(i->bp >= i->blen) {
		if(i->bp > 0)
			printf(".");
		i->bp = 0;
		if((i->blen = read(i->fd, i->buf, sizeof i->buf)) <= 0) {
			printf("truncated\n");
			return -1;
		}
		/*
		int n;
		for(n=0; n<i->blen; n += 512) 
			printf("%lx-%lx: crc: %lx %lx\n",
				lseek(i->fd, 0, SEEK_CUR)-i->blen+n,
				lseek(i->fd, 0, SEEK_CUR)-i->blen+n+511,
				updcrc(i->buf+n, 512, 0),
				gp->gz_crc);
		*/
	}
	return i->buf[i->bp++];
}

static void
eread(Ibuf *i, byte *p, int len)
{
	while(len--) {
		int c = lgetc(i);
		if(c < 0)
			exit(1); 
		*p++ = c;
	}
}

static int
unzip(fd_t ifd, fd_t ofd)
{
	struct inflate g;
	ushort magic;
	int flags;
	byte b[10];
	sz name[256];
	ulong crc, len;
	off_t s0 = lseek(ifd, 0, SEEK_CUR);
	Ibuf inp;

	inp.blen = 0;
	inp.bp = 0;
	inp.fd = ifd;
	g.gz_id = &inp;
	g.gz_od = (void*)ofd;
	g.gz_getc = (int (*)(void*))lgetc;
	g.gz_write =  (int (*)(void*, const byte*, int))write;
	g.gz_slide = (byte*)malloc(GZ_WSIZE);
	g.gz_crc = 0;
	g.gz_len = 0;
gp = &g;

	eread(&inp, b, 10);
	magic = (b[1] << 8) | b[0];
	if(magic != 0x8b1f && magic != 0x9e1f) {
		printf("bad magic: %x\n", magic);
		exit(1);
	}
	if(b[2] != 8)	{
		printf("only 'deflated' is supported\n");
		exit(1);
	}
	if((flags = b[3]) & 0xe2) {
		printf("unsupported flags\n");
		exit(1);
	}
	if(flags & 0x04) {	/* extra field */
		eread(&inp, b, 2);
		len = (b[1] << 8) | b[0];
		while(len--)
			eread(&inp, b, 1);
	}
	if(flags & 0x08) {	/* original name */
		sz *n = name;
		while((*n++ = lgetc(&inp)))
			;
	} else
		name[0] = 0;
	if(flags & 0x10) {	/* comment */
		while(lgetc(&inp))
			;
	}
	if(ofd != STDOUT_FILENO)
		printf("Inflating %s ", name);
	inflate(&g);
	eread(&inp, b, 8);
	
	crc = (b[3]<<24)|(b[2]<<16)|(b[1]<<8)|b[0];
	len =  (b[7]<<24)|(b[6]<<16)|(b[5]<<8)|b[4];
	if(g.gz_crc != crc || g.gz_len != len) {
		printf("error crc:%lx/%lx len:%lx/%lx pos:%lx\n", g.gz_crc, crc,
				g.gz_len, len, lseek(ifd, 0, SEEK_CUR)-s0);
		exit(1);
	}
	if(ofd != STDOUT_FILENO)
		printf("%ld -> %ld\n", lseek(ifd, 0, SEEK_CUR)-s0, g.gz_len);
	return g.gz_len;
}


int
unzip(int argc, csz **argv)
{
	fd_t ifd;
	fd_t ofd;
	ulong rdbase = 0;
	ulong rdsize = 0;

	if(argc < 2 || argc > 3) {
		printf("usage: %s [-v] [-r] <src> [<dest>]\n", argv[0]);
		return 1;
	}
	if(strcmp(argv[1], "-v") == 0) {
		printf("unzip " unzip_VER_STR "\n");
		return 0;
	}
	if(strcmp(argv[1], "-r") == 0) {
		rdsize = 0x400000;
		rdbase = bootparam->ext_addr+bootparam->ext_size-rdsize;
		rdbase = rdbase&0xffff0000;
		++argv;
		--argc;
	}
	if((ifd = open(argv[1], O_RDONLY)) < 0) {
		printf("could not open %s\n", argv[1]);
		return 1;
	}
	if(argc > 2 || rdbase > 0) {
		sz buf[50];
		csz *fn = argv[2];
		if(rdbase > 0) {
			sprintf(buf, "@%lx\n", rdbase);
			fn = buf;
		}
		if((ofd = open(fn, O_WRONLY)) < 0) {
			printf("could not open %s\n", fn);
			return 1;
		}
	} else 
		ofd = STDOUT_FILENO;

	unzip(ifd, ofd);
	close(ifd);
	if(ofd != STDOUT_FILENO)
		close(ofd);

	if(rdbase) {
		if(sz *p = getenv("ramdiskbase"))
			sprintf(p, "0x%08lx", rdbase);
		if(sz *p = getenv("ramdisksize"))
			sprintf(p, "0x%08lx", rdsize);
		if(sz *p = getenv("*maxmem"))
			sprintf(p, "0x%08lx", rdbase);
	}
	return 0;
}

