/*  $Id: diskmap-Linux.c,v 1.2 2000/01/12 03:44:47 shaggy Exp $
 *      configure a disk to use SOLO
 *      
 *      relative path: sys/boot/m-pc
 *  
 *  Copyright (c) 1994-1999 Frank E. Barrus.  All rights reserved.
 *  
 *  This file is part of the ShagOS distribution, and may be freely used and
 *  distributed by the terms of the ShagWare General Public License (SGPL),
 *  version 2, or, at your option, any newer official version of that License.
 *  
 *  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT WARRANTY, EXPRESS OR IMPLIED.
 *  THE AUTHOR SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF ANY KIND,
 *  INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 *  FITNESS FOR A PARTICULAR PURPOSE, AND WILL NOT BE LIABLE FOR ANY DAMAGES.
 *  For more warranty details, see the ShagWare General Public License, which
 *  should have been received with this program, in a file called 'lic_spl2.txt'
 *  If this license is missing, please contact the author and describe how
 *  this software was obtained.  Email: shaggy@csh.rit.edu
 *  
 *  Complete, unaltered, original distributions of this and related code
 *  may be obtained from:   http://www.csh.rit.edu/~shaggy/software.html
 *  or:	ftp://ftp.csh.rit.edu/pub/members/shaggy
 */

#define _POSIX_SOURCE
#include <libc.h>
#include <stdx1.h>
#include <dirent.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <linux/major.h>
#define __ARCH_I386_ATOMIC__	/* cause exclusion to avoid warnings */
#define _LINUX_BITOPS_H		/* cause exclusion to avoid warnings */
#include <linux/fs.h>
#include <linux/fd.h>
#include <linux/hdreg.h>
#include <boot/mapfile.h>

#include "diskmap.h"

#ifndef SCSI_DISK0_MAJOR
#	define SCSI_DISK0_MAJOR SCSI_DISK_MAJOR
#endif


int
devname(dev_t dev, sz *devname)
{
	DIR* dir;
	struct stat stbuf;
	struct dirent* ent;

	dir = opendir("/dev");
	devname[0] = '\0';
	while((ent = readdir(dir))) {
		sprintf(devname, "/dev/%s", ent->d_name);
		if(stat(devname, &stbuf) < 0)
			return -1; 
		if(S_ISBLK(stbuf.st_mode) && stbuf.st_rdev == dev)
			break; 
		devname[0] = '\0';
	}
	closedir(dir);
	if(!devname[0]) {
		errno = ENODEV;
		return -1;
	}
	return 0;
}

int
diskinfo(int fd, struct diskinfo *di)
{
	struct stat stbuf;
	struct floppy_struct fdgeo;
	struct hd_geometry hdgeo;

	if(fstat(fd, &stbuf) < 0) 
		return -1;
	if(!S_ISBLK(stbuf.st_mode)) {
		errno = ENOTBLK;
		return -1;
	}
	di->devid = stbuf.st_rdev;
	di->secsize = 512;
	switch(MAJOR(stbuf.st_rdev)) {
	case FLOPPY_MAJOR:
		if(ioctl(fd, FDGETPRM, &fdgeo) < 0) 
			return -1;
		di->heads = fdgeo.head;
		di->cyls = fdgeo.track;
		di->secs = di->maxmult = fdgeo.sect;
		di->start = 0;
		di->biosid = 0x00 | (stbuf.st_rdev & 0x0f);
		di->tsecs = di->cyls*di->heads*di->secs;
		break;
	case HD_MAJOR:
#ifdef IDE1_MAJOR
	case IDE1_MAJOR:
#endif
#ifdef IDE2_MAJOR
	case IDE2_MAJOR:
#endif
#ifdef IDE3_MAJOR
	case IDE3_MAJOR:
#endif
#ifdef IDE4_MAJOR
	case IDE4_MAJOR:
#endif
#ifdef IDE5_MAJOR
	case IDE5_MAJOR:
#endif
	case XT_DISK_MAJOR:
	case SCSI_DISK0_MAJOR:
		if(ioctl(fd, HDIO_GETGEO, &hdgeo) < 0) 
			return -1;
		di->heads = hdgeo.heads;
		di->cyls = hdgeo.cylinders;
		di->secs = di->maxmult = hdgeo.sectors;
		di->start = hdgeo.start;
		if(MAJOR(stbuf.st_rdev) == SCSI_DISK0_MAJOR)
			di->biosid = 0x80 | (MINOR(stbuf.st_rdev) >> 4); 
		else
			di->biosid = 0x80 | (MINOR(stbuf.st_rdev) >> 6); 
		di->tsecs = di->cyls*di->heads*di->secs;
		break;
	default:
		/* perhaps this can be made to support linear mode for unknown devices */
		errno = ENOTBLK;
		return -1;
	}
	return 0;
}

/*
 * call initially with dm->pos == -1, and
 * this will initialize both structures,
 * and set dm->pos to 0.
 */
int
diskmap(int fd, struct diskinfo *di, struct diskmap *dm)
{
	unsigned int blocksize;
	unsigned long block, sec;
	struct stat stbuf;
	off_t ofs;
	size_t len;

	if(dm->pos == -1) {
		if(fstat(fd, &stbuf) < 0) 
			return -1;
		dm->size = stbuf.st_size;
		if(!S_ISBLK(stbuf.st_mode)) {
			int devfd;
			sz path[_POSIX_PATH_MAX];
			if(devname(stbuf.st_dev, path) < 0)
				return -1;
			if((devfd = open(path, O_RDWR)) < 0)
				return -1;
			if(diskinfo(devfd, di) < 0) {
				close(devfd);
				return -1;
			}
			close(devfd);
			if(ioctl(fd, FIGETBSZ, &dm->blocksize) < 0)
				return -1;
		} else {
			if(diskinfo(fd, di) < 0)
				return -1;
			if(dm->size == 0)
				dm->size = di->tsecs*di->secsize;
			dm->blocksize = 0;
		}
		dm->pos = 0;
		return 0;
	}
	if(dm->blocksize) {
		blocksize = dm->blocksize;
		block = dm->pos/blocksize;
		if(ioctl(fd, FIBMAP, &block) < 0)
			return -1;
	} else {
		blocksize = di->secsize;
		block = dm->pos/blocksize;
	}
	sec = block*(blocksize/di->secsize);
	ofs = dm->pos % blocksize;
	len = blocksize - ofs;
	sec += di->start+ofs/di->secsize;
	ofs %= di->secsize;
	if(len > dm->size - dm->pos)
		len = dm->size - dm->pos;
	dm->linear = sec;
	dm->cyl = sec / (di->secs*di->heads);
	sec -= dm->cyl*di->secs*di->heads;
	dm->head = sec / di->secs;
	dm->sec = sec - dm->head*di->secs + 1;
	dm->nsec = (len+ofs+di->secsize-1)/di->secsize;
	if(dm->sec+dm->nsec-1 > di->secs) {
		dm->nsec = di->secs-dm->sec+1;
		len = di->secsize*dm->nsec - ofs;
	}
	dm->len = len;
	dm->ofs = ofs;
	if(block == 0 && dm->blocksize) {
		dm->linear = 0;
		dm->cyl = dm->head = dm->sec = 0;
	}
	return 0;
}

