/*  $Id: readmap.cc,v 1.3 1999/12/18 21:29:13 shaggy Exp $
 *      Read and display SOLO mapfiles	
 *      relative path: ./sys/boot/m-pc
 */

#include <libc.h>
#include <stdx1.h>
#include <boot/mapfile.h>

#include "../boot.h"

static void
readmap(FILE *f)
{
	sz str[256];
	MapFile mapfile;
	Chunk chunk;
	int i;
	int adr;

	while(fread(&mapfile, sizeof(mapfile), 1, f) && mapfile.namelen) {
		fread(str, 1, mapfile.namelen, f);
		printf("%s: nchunks=%d size=%ld dev=0x%02X unit=0x%04X\n", str,
			mapfile.nchunks, mapfile.fsize, mapfile.device,
			mapfile.unit);
		adr = 0;
		for(i=0; i<mapfile.nchunks; i++) {
			fread(&chunk, sizeof(chunk), 1, f);
			if(chunk.partial)
				printf(
			 "  %d (%x):  cyl:%d  head:%d  sec:%d  %x:%x\n",
					i, adr, (chunk.cyl1)|(chunk.cyl2<<8),
					chunk.head, 
					chunk.sector, chunk.count, chunk.count);
			else
				printf(
			 "  %d (%x):  cyl:%d  head:%d  sec:%d  count:%d\n",
			   		i, adr, (chunk.cyl1)|(chunk.cyl2<<8),
					chunk.head,
					chunk.sector, chunk.count);
			adr += (SECTOR_SIZE*chunk.count);
		}
	}
}

static void
readboot(FILE *f)
{
	BootBlock b;
	int i;
	int adr;

	if(fread(&b, sizeof b, 1, f) != 1) {
		perror("bootsec");
		return;
	}
	adr = 0;
	printf("mapdev:%x mapsize:%d bootoffset=%x\n", 
		b.mapdev, b.mapsize, (int)b.bootoffset);
	for(i=0; i<b.mapsize; i++) {
		Chunk *c = &b.chunk[i];
		if(c->partial)
			printf(
		 "  %d (%x):  cyl:%d  head:%d  sec:%d  %x:%x\n",
				i, adr, (c->cyl1)|(c->cyl2<<8),
				c->head, 
				c->sector, c->count, c->count);
		else
			printf(
		 "  %d (%x):  cyl:%d  head:%d  sec:%d  count:%d\n",
		   		i, adr, (c->cyl1)|(c->cyl2<<8),
				c->head, c->sector, c->count);
		adr += (SECTOR_SIZE*c->count);
	}
}

int
main(int argc, csz **argv)
{
	int i = 0;
	FILE *f;

	if(argc > 1 && strcmp(argv[1], "-b") == 0) {
		if(argc == 2)
			readboot(stdin);
		else if(!(f = fopen(argv[2], "r")))
			perror(argv[2]);
		else {
			readboot(f);
			fclose(f);
		}
		return 0;
	}

	if(argc == 1)
		readmap(stdin);
	while(++i < argc) {
		if(!strcmp(argv[i], "-"))
			readmap(stdin);
		else if(!(f = fopen(argv[i], "r"))) 
			perror(argv[i]);
		else {
			readmap(f);
			fclose(f);
		}
	}
	return 0;
}

